function [ returnvalue ] = siddon2D( L1, L2, N, theta, z )
%   raytracing: 
%       siddon2D( L1, L2, N, theta, z )
%       this function calculate the ray tracing projection
%       matrix elements along a certain ray with Siddon's algorithm
%       it works for 2D case only
%   Input:
%       L1    :   distance from isocenter to source, in pixel size
%       L2    :   distance from isocenter to the center of the detector, in
%                 pixel size
%       N     :   dimension of the image
%       theta :   projection angle counting from positive x direction, in rad 
%       z     :   the coordinate along imager, origining at the center, in
%                 unit of pixel size
%   Output:
%       value :   sparse representation of the projection matrix for the
%                 given ray
%   Note:
%       in the output, the index of pixels is in 1d order. the pixel is labeled
%       in this way. i = 1:N*N; i = reshape(i,N,N); then the corresponding
%       element is given by value of i

    x1 = L1*cos(theta);
	y1 = L1*sin(theta);

	x2 = z*sin(theta) - L2*cos(theta);
	y2 = -z*cos(theta) - L2*sin(theta);
%	compute the source and detector coordinate
	
    flagX = 0;
    flagY = 0;
    i = 1:N+1;
	if (x2 < x1)
		alphaX = (-N/2 + (N+1-i) - x1)/ (x2-x1);
    elseif (x2 > x1)
		alphaX = (-N/2 + i-1 - x1) / (x2-x1);
    else
        flagX = 1;
    end
    if (y2 < y1)
        alphaY = (-N/2 + (N+1-i) - y1) / (y2-y1);
    elseif (y2 > y1)
		alphaY = (-N/2 + i-1 - y1) / (y2-y1);
    else
        flagY = 1;
    end
%	construct full list for both directions

    if (flagX == 1)
        alphaMin = max([0.0 alphaY(1)]);
    elseif (flagY == 1)
        alphaMin = max([0.0 alphaX(1)]);
    else
        alphaMin = max([0.0 alphaX(1) alphaY(1)]);
    end
    if (flagX == 1)
        alphaMax = min([1.0 alphaY(N+1)]);
    elseif (flagY == 1)
        alphaMax = min([1.0 alphaX(N+1)]);
    else
        alphaMax = min([1.0 alphaX(N+1) alphaY(N+1)]);
    end
%	find out the range, alphamax and alphamin
    
    if (flagX == 1)
        alpha = alphaY;
    elseif (flagY == 1)
        alpha = alphaX;
    else
        alpha = [alphaX alphaY];
    end
    
    indx = find(((alpha <= alphaMax) & (alpha >= alphaMin)));
    alpha = alpha(indx);
	alpha = sort(alpha);
%	merge into a single list and remove unnecessary iterms

	d12 = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
%	full lengh between source and deterctor
		
    value = (alpha(2:end)-alpha(1:end-1))*d12;
    
    alphaMid = (alpha(2:end) + alpha(1:end-1))/2.0;

    tempx = floor(x1 + alphaMid * (x2 - x1) +N/2) +1;
    tempy = floor(y1 + alphaMid * (y2 - y1) +N/2) +1;

	j = (tempx-1)*N + tempy;
    i = ones(1,length(j));
%	for each ray compute each voxel position and corresponding weight

	returnvalue = sparse ( i, j, value, 1, N*N);
    
end

